********************************************************************************
***       Module de gestion des interruption et de l'afficheur 7 segments    ***
********************************************************************************

	include declare.s
	xref	initgen,initpit
	xdef	partie
	xdef	initirq
	xdef	initscroll
	xdef	temps


	xref	nouveau
	xref	box_status

	sect prog,,c
	

********************************************************************************
***             Cette fonction initialise les differentes IRQ                ***
***  Parametres d'entree:	 Aucun.					     ***
***  Parametres de sorties:	 Aucun.					     ***
***  Registres   Modifiees:	 Tous sauf a7				     ***
********************************************************************************			
initirq
	jsr	initgen
	move.b	#0,D0
	jsr	initpit
	move.l	#credit_msg,memaffiche ;Chargement des credits.
	jsr	initinter	       ;Initialisation de l'inter.
	jsr	initscroll	       ;Initialisation du scrolling.
	jsr	inittimer	       ;Initialisation du timer.
	jsr	initrtc		       ;Initialisation du RTC.
	rts

********************************************************************************
***            Cette fonction convertit un nombre au format afficheur        ***
*** Parametres d'entrees: a6.l: adresse ou stockee les 3 chiffres du resultat***
***			  d5.l: Nombre a convertir.			     ***	
*** Parametres de sorties: Aucun					     ***
*** Registres modifiees:   a5,a6,d5,d6,d7				     ***
********************************************************************************
nombre
	move.l	#chiffres,a5	      ;Chargement des chiffres.
	add.l	#3,a6		      ;On commence par le chiffre le + a droite.
	move.b	#3,d7
divise
	divu	#10,d5		       
	swap	d5
	move.b	(a5,d5),-(a6)	      ;Stockage du reste. 
	clr.w	d5		     
	swap	d5
	sub.b	#1,d7
	bne	divise				
	rts
	
********************************************************************************
***            Cette fonction initialise le scrolling			     ***
*** Parametres d'entrees:   Aucun.					     ***
*** Parametres de sorties:  Aucun.					     ***
*** Registres modifiees:  d0.l						     ***	
********************************************************************************
initscroll
	move.l	#0,decalescroll	     ;Initialisation de toutes les variables.
	move.w	#0,memafcour
	move.b	#1,afcourant
	move.b	#2,delaiscroll
	move.b	#0,eteint
	move.b	#1,scroll
	move.b	#0,clignote
	move.b	#0,delai
	clr.l	d0		     ;Effacement des afficheurs.
	not.l	d0
	move.l	d0,memvid
	move.l	d0,memvid+4
	rts
	
********************************************************************************
***             Cette fonction initialise l'interruption Timer               ***
*** Parametres d'entrees:   Aucun.					     ***
*** Parametres de sorties:  Aucun.				             ***
*** Registres modifiees:  Auncun.					     ***
********************************************************************************
inittimer
	bclr.b	#0,tcr
	move.b	#0,cprh
	move.b	#02,cprm
	move.b	#0,cprl
	move.l	#timerirq,4*vectimer
	move.b	#vectimer,tivr
 	move.b	#$A1,tcr
	rts

********************************************************************************
***              Cette fonction initialise l'interruption rtc		     ***
*** Parametres d'entrees:   Aucun.					     ***
*** Parametres de sorties:  Aucun.					     ***
*** Registres modifiees:  Auncun.					     ***
********************************************************************************
initrtc
	move.l	#rtcirq,vectrtc
	bset.b	#1,icr
	rts

********************************************************************************
***       Cette fonction initialise l'interruption de l'interrupteur         ***
*** Parametres d'entrees:   Aucun.				             ***
*** Parametres de sorties:  Aucun.					     ***
*** Registres modifiees:  Auncun.					     ***
********************************************************************************
initinter
	move.l	#interirq,4*vecpit
	move.b 	#vecpit,pivr
	bset.b	#1,pacr
	rts

********************************************************************************
***              Cette fonction fait scroller l'afficheur.                   ***
*** Parametres d'entrees:   Aucun.					     ***
*** Parametres de sorties:  Aucun.				             ***
*** Registres modifiees:  Aucun.					     ***
********************************************************************************
fait_scroll
	move.l	#memvid,a6	
	move.b	1(a6),(a6)+		;Decalement de la zone affichee
	move.b	1(a6),(a6)+		;D'un octet vers la gauche.
	move.b  1(a6),(a6)+
	move.b  1(a6),(a6)+	
	move.b	1(a6),(a6)+
	move.b	1(a6),(a6)+
	move.b	1(a6),(a6)+
	move.b  1(a6),(a6)+
	move.b	d6,memvid+7
	rts
	
********************************************************************************
***          Cette fonction affiche un texte sur l'afficheur 7 segments	     ***
*** Parametres d'entrees: Aucun.					     ***
******************************************************************************** 
affiche_scroll
	move.l	a0,a6
	add.l	#2,a6			
	move.b	(a6)+,memvid+0		
	move.b	(a6)+,memvid+1
	move.b	(a6)+,memvid+2
	move.b	(a6)+,memvid+3
	move.b	(a6)+,memvid+4
	move.b	(a6)+,memvid+5
	move.b	(a6)+,memvid+6
	move.b	(a6)+,memvid+7
	add.l	#8,decalescroll		
	rts

********************************************************************************
***       Cette fonction affiche le temps restant pour le niveau en cours    ***
*** Parametres d'entrees: Aucun.                                             ***
*** Parametres de sorties: Aucun.					     ***
*** Registres modifiees:	a5,a6,d5,d5,d6,d7			     ***
********************************************************************************
affiche_temps
	move.l	#memvid,a6		;Effacement des afficheurs.
	move.l	#$FFFFFFFF,(a6)
	move.b	#$FF,4(a6)
	add.l	#5,a6
	clr.l	d5
	move.w	temps,d5
	lsr.w	#3,d5
	jsr	nombre			;Affichage du nouveau nombre.
	rts		

********************************************************************************
***     Cette fonction interprete le texte du scrolling.		     ***
*** Ce texte se situe a l'adresse memaffiche.				     ***
*** Le premier octet indique la longueur du texte.			     ***
*** Les octets suivants ont la structure suivante:			     ***
*** [76543|2|1|0]							     ***
*** Si le bit 0 est a 1, alors l'octet est affiche.			     ***
*** Si le bit 0 est a 0, alors les bits suivants ont la signification        ***
***		suivantes:						     ***
***	Le bit 1 a 1 indique le clignotement du scrolling.		     ***
***	Le bit 2 a 1 indique l'affichage sans scrolling des 8 caracteres     ***
***		suivants						     ***
***	Les bits 2-7 indiquent une pause de n iterations avant la reprise du ***
***		scrolling.						     ***
*** Parametres d'entrees: Aucun.					     ***
*** Parametres de sorties: Aucun.					     ***
*** Registres modifiees: tous.						     ***
********************************************************************************
prepare_scroll
	and.b	#1,clignote		;Test si il faut faire clignoter.
	beq	clignote_pas
	add.b	#1,eteint		;Oui, on ajoute 1 a eteint.
	bra	suite4
clignote_pas
	clr.b	eteint			;Non on remet eteint a 0.
suite4
	sub.b	#1,delaiscroll		;Doit-on lire le prochain caractere ?.
	bne	pas_scroll		
	move.b	#2,delaiscroll		;Oui, on reinitialise delaiscroll.
*Interpretation des caracteres de commande.
	move.l	memaffiche,a0		
	move.l	decalescroll,d0
	clr.l	d1
	move.b	(a0),d1
	divu	d1,d0
	clr.w	d0
	swap	d0
	add.l	d0,a0
	move.b	1(a0),d6		;Lecture du caractere suivant:
	btst.b	#0,d6			
	bne	pas_special
	move.b	1(a0),d0		;Caractere special.
	lsr.b	#1,d0
	move.b	d0,clignote		;Chargement de clignote.
	lsr.b	#1,d0			
	btst.b	#0,d0			;Doit-on affiche immediatement
					;les 8 caracteres suivant?
	beq	suite10			;Non.
	jsr	affiche_scroll		
suite10
	lsr.b	#1,d0			;Pendant combien de temps 
	move.b	d0,delaiscroll		;Le texte doit-il etre affiche
	add.l	#1,decalescroll		;sans scrolle.
	bra	pas_scroll
	move.b	2(a0),d6
pas_special
	and.b	#1,scroll		;Le texte doit-il defile???
	beq	pas_scroll		;Oui.
	jsr	fait_scroll
	add.l	#1,decalescroll	
pas_scroll
	rts

********************************************************************************
***     Cette fonction est appele lors de l'appui sur le bouton poussoir.    ***
********************************************************************************
interirq
	or	#%0000011000000000,sr	;Masque d'interruption.
	and	#%1111111011111111,sr	;Niveau 6.
	movem.l	d0-d7/a0-a6,-(sp)	;Sauvegarde des registres.
	btst.b	#4,psr			;Le bouton est-il presse?
	bne	bouton_relache
	cmp.b	#5,partie		;La partie est-elle finie ?
	beq	bouton_relache		
	cmp.b	#0,partie		;Existe t'il un partie en cours? 
	beq	suite80			;Non.
	cmp.b	#0,box_status		;Doit-on resetter le niveau
					;Par interruption?
	beq	suite90			;Non.
	jsr	nouveau			
	bra	bouton_relache
suite90
	move.b	#2,partie		;Demande de reset par le module box.
	bra	bouton_relache
suite80
	move.b	#1,partie		;Partie en cours.
	jsr	nouveau			;Reset de la partie.
	jsr	initscroll		;Reset du scrolling.
bouton_relache
	movem.l (sp)+,d0-d7/a0-a6	;Restauration des interruptions.
	bset.b	#0,psr
	rte
	
********************************************************************************
*** Cette fonction est appelee tous les dixiemes de secondes par la RTC      ***
********************************************************************************
rtcirq
	or	#%0000011000000000,sr	;Niveau 6
	and	#%1111111011111111,sr 
	movem.l	d0-d7/a0-a6,-(sp)	;Sauvegarde des interruptions.
	cmp.b	#0,partie		;Existe t'il une partie en cours.
	beq	pas_partie		;Non.
	cmp.b	#3,partie		;Doit on affiche le message de
					; changement de niveau.
	bne	pas_gagne		;Non.
	move.b	#0,partie		;Fin de la partie
	move.l	#change_niveau_msg,memaffiche
	jsr	initscroll
	bra	suite20
pas_gagne				
	cmp.b	#4,partie		;Affichage du message de la fin du jeu?
	bne	pas_g_jeu		;non.
	move.b	#5,partie		;Jeu fini.
	move.l	#fini_msg,memaffiche
	jsr	initscroll
	bra	pas_partie
pas_g_jeu
	cmp.b	#5,partie		;Le jeu est-il fini?
	beq	pas_partie
	sub.w	#1,temps
	bne	suite60			;Plus de temps?
	move.b	#0,partie
	move.l	#timeout_msg,memaffiche	;Affichage du message de Time out.
	jsr	initscroll		
	bra	suite20			
suite60
	jsr	affiche_temps
	bra	suite20
pas_partie
	jsr	prepare_scroll		;Scrolling.
suite20
	movem.l	(sp)+,d0-d7/a0-a6
	bset.b	#0,isr
	rte

********************************************************************************
*** Cette fonction est appelee par l'interruption timer.	             ***
********************************************************************************
timerirq
	or	#%0000011000000000,sr	;Niveau 6.
	and	#%1111111011111111,sr
	movem.l	d0-d7/a0-a6,-(sp)
	move.b	afcourant,d0		;Masque pour les afficheurs.
	move.b	d0,pbdr 
	move.b	#$FF,padr		;Effacement de l'ancien afficheur.
	ror.b	#1,d0
	btst.b	#1,eteint		;Doit-on utilise l'afficheur.
	bne	pas_affiche
	move.l	#memvid,a0		
	move.w	memafcour,d1		;Affiche le caractere suivant.
	move.b	d0,pbdr
	move.b  (a0,d1),padr
pas_affiche
	move.b	d0,afcourant	
	add.w	#1,memafcour
	and.w	#$7,memafcour
	movem.l	(sp)+,d0-d7/a0-a6
	bset.b	#$0,tsr
	rte

	
	

	
	sect data,,d
*$25: S
*$8D: C
*$81: 0
*$8F: L
*$11: A
*$5F: r
*$1F: t
*$F3: I
*$0D: E
*$91: N
*$83: U
*$13: H
*$05: G
*$43: d
*$07: b
*$23: y
*$C3: J

	
***   Les credits!!!!!!! ***
credit_msg	  DC.B $4A,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
		  DC.B $25,$81,$8D,$81,$07,$11,$91,$FF,$F2,$08 
		  DC.B $FF,$FF,$07,$23,$FF,$FF,$FF,$FF
		  DC.B $7F,$13,$11,$91,$25,$7F,$FF,$F2,$08
		  DC.B $11,$91,$81,$FF,$FF,$FF,$FF
		  DC.B $25,$83,$91,$FF,$FF,$F2,$08
		  DC.B $84,$FF,$F3,$91,$25,$0D,$5F,$1F,$FF
		  DC.B $84,$FF,$FF,$8D,$81,$F3,$91,$FF,$FF,$0C
		  DC.B $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$08
		
		  DC.B $25,$4F,$81,$5F,$0D,$FF

***  Message affiche lors du changement de niveau. ***
change_niveau_msg DC.B $2D
		  DC.B $8F,$0D,$83,$0D,$8F,$FF,$FF,$8D,$8F,$0D,$11,$5F,$FF,$F2
		  DC.B $08,$F4,$0D,$91,$1F,$0D,$5F,$F3,$91,$05,$FF,$1F,$81,$FF
		  DC.B $91,$0D,$13,$1F,$FF
		  DC.B $8F,$0D,$83,$0D,$8F,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF

*** Message affiche lors du Time out.
timeout_msg       DC.B $09
		  DC.B $F6,$1F,$F3,$91,$0D,$FF,$81,$83,$1F

*** Message affiche lors de la fin du jeu.
fini_msg	  DC.B 78
		  DC.B $05,$81,$81,$43,$FF,$C3,$81,$07,$F2,$08,$FF
		  DC.B $23,$81,$83,$FF,$13,$11,$83,$0D,$FF
		  DC.B $1D,$F3,$91,$F3,$25,$13,$0D,$43,$FF
		  DC.B $25,$81,$8D,$81,$07,$11,$91,$FF,$F2,$08
		  DC.B $11,$FF,$05,$11,$91,$0D,$FF,$07,$23,$FF
		  DC.B $7F,$13,$11,$91,$25,$7F,$FF,$F2,$08
		  DC.B $11,$91,$81,$FF,$FF,$FF,$FF
		  DC.B $25,$83,$91,$FF,$FF,$F2,$08
		  DC.B $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
		  DC.B $FF,$FF,$FF,$FF,$FF,$FF
	
*** Chiffres pour l'afficheur ***
chiffres	  DC.B	%10000001	0
		  DC.B	%11110011	1
		  DC.B	%01001001	2
		  DC.B	%01100001	3
		  DC.B	%00110011	4
		  DC.B	%00100101	5
		  DC.B	%00000101	6
		  DC.B	%11110001	7
		  DC.B	%00000001	8
		  DC.B	%00100001	9

*** Memoire representant les 8 afficheurs***
memvid		  DS.L 2

memaffiche	  DS.L 1		; Debut de la chaine a afficher.
afcourant	  DS.B 1		; Afficheur courant.
memafcour	  DS.W 1		; Caractere courant a affiche.
decalescroll	  DS.L 1		; Compteur de decalage.
delaiscroll	  DS.B 1
eteint		  DS.B 1	 
scroll		  DS.B 1
clignote	  DS.B 1
delai		  DS.B 1		;Delai.

temps		  DS.W 1
partie		  DS.B 1
